
# q-expansion of newform 7920.2.a.r, downloaded from the LMFDB on 16 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7920
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-1], [4], [1], [-2], [2], [8], [0], [-2], [8], [-10], [10], [0], [0], [-14], [-4], [14], [4], [8], [10], [-12], [4], [6], [-14], [6], [-8], [-4], [-10], [6], [12], [-12], [-18], [0], [-2], [-4], [-2], [12], [24], [22], [12], [6], [-16], [10], [-2], [0], [24], [-16], [4], [-10], [2], [-24], [26], [-12], [-18], [8], [10], [28], [-10], [-6], [16], [6], [0], [32], [-22], [18], [-12], [2], [-4], [-2], [14], [24], [0], [-34], [20], [24], [-22], [-18], [38], [-14], [-36], [-26], [24], [-14], [4], [-4], [14], [10], [-26], [-32], [20], [40], [-16], [20], [4], [24], [42], [-2], [32], [38], [-8], [-34], [-36], [18], [-8], [-14], [44], [42], [0], [10], [12], [-2], [30], [4], [32], [-34], [44], [16], [-22], [36], [-10], [10], [46], [-12], [-36], [6], [-10], [-16], [-16], [14], [-16], [-32], [-8], [-42], [-30], [34], [-14], [40], [34], [-30], [0], [-26], [24], [12], [-2], [-16], [14], [42], [-28], [-24], [6], [54], [4], [-48], [-28], [-16], [20], [6], [-14], [30], [-36], [-6], [-60], [-36], [54], [-32], [-40], [14], [26], [-30], [-20], [30], [-8], [-22], [-32], [-2], [8], [-30], [-28], [14], [-40], [-12], [-26], [-6], [0], [-30], [-50], [-4], [50], [-24], [2], [-20], [20], [-62], [-20], [-42], [-14], [30], [18], [0], [-26], [52], [-10], [-22], [-12], [-30], [16], [-36], [-46], [-36], [26], [-14], [16], [60], [24], [26], [-44], [-14], [8], [-54], [-50], [44], [-10], [32], [-36], [-58], [-50], [-24], [-52], [-28], [-50], [40], [52], [-46], [-52], [-8], [2], [38], [68], [0], [-28], [56], [-32], [14], [10], [-56], [-24], [-20], [-56], [-24], [-2], [-30], [72], [-70], [-66], [-4], [-10], [-8], [42], [66], [-12], [28], [-50], [-42], [14], [52], [74], [-18], [-24], [46], [46], [76], [-70], [-36], [-30], [-16], [20], [46], [42], [-12], [64], [16], [-48], [54], [32], [-40], [2], [-2], [64], [-78], [66], [-28], [34], [-20], [-26], [22], [-56], [-30], [4], [-64], [-62], [-66], [20], [36], [-68], [50], [-12], [46], [-32], [-2], [16], [-70], [-30], [-12], [72], [-14], [-12], [0], [2], [-22], [8], [-22], [-42], [-24], [-30], [50], [-28], [44], [-48], [-34], [-26], [-62], [-44], [52], [-40], [-84], [30], [-6], [-22], [-20], [54], [-58], [30], [-8], [-6], [-68], [22], [68], [48], [2], [-56], [74], [-54], [-4], [-26], [26], [-80], [-76], [2], [8], [62], [-54], [-88], [52], [4], [26], [30], [52], [42], [52], [-12], [-24], [22], [-28], [-2], [76], [8], [10], [-86], [26], [18], [-26], [68], [2], [-64], [-40], [-8], [-42], [80], [-48], [-30], [-46], [100], [-76], [-32], [-86], [68], [22], [100], [6], [14], [-10], [-20], [38], [-34], [44], [-50], [18], [-12], [28], [26], [6], [36], [-32], [26], [-78], [-8], [88], [14], [-8], [-6], [-82], [16], [84], [106], [42], [-36], [-54], [52], [0], [10], [-4], [-4], [-32], [-34], [38], [2], [-10], [20], [-68], [28], [102], [70], [0], [26], [-102], [56], [0], [-78], [6], [0], [96], [-44], [66], [34], [90], [-58], [-20], [22], [30], [92], [-24], [-60], [-26], [-56], [-110], [84], [84], [82], [92], [68], [-92], [-40], [-22], [60], [46], [10], [80], [-40], [74], [90], [-30]]
